﻿using System;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace Spectrum
{
    public partial class frmMain : Form
    {

        public enum mir_sdr_ErrT
        {
          mir_sdr_Success            = 0,
          mir_sdr_Fail               = 1,
          mir_sdr_InvalidParam       = 2,
          mir_sdr_OutOfRange         = 3,
          mir_sdr_GainUpdateError    = 4,
          mir_sdr_RfUpdateError      = 5,
          mir_sdr_FsUpdateError      = 6,
          mir_sdr_HwError            = 7,
          mir_sdr_AliasingError      = 8,
          mir_sdr_AlreadyInitialised = 9,
          mir_sdr_NotInitialised     = 10,
          mir_sdr_NotEnabled         = 11,
          mir_sdr_HwVerError         = 12,
          mir_sdr_OutOfMemError      = 13
        } 

        public enum  mir_sdr_Bw_MHzT
        {
          mir_sdr_BW_Undefined = 0,
          mir_sdr_BW_0_200     = 200,
          mir_sdr_BW_0_300     = 300,
          mir_sdr_BW_0_600     = 600,
          mir_sdr_BW_1_536     = 1536,
          mir_sdr_BW_5_000     = 5000,
          mir_sdr_BW_6_000     = 6000,
          mir_sdr_BW_7_000     = 7000,
          mir_sdr_BW_8_000     = 8000
        }

        private enum mir_sdr_If_kHzT
        {
          mir_sdr_IF_Undefined = -1,
          mir_sdr_IF_Zero      = 0,
          mir_sdr_IF_0_450     = 450,
          mir_sdr_IF_1_620     = 1620,
          mir_sdr_IF_2_048     = 2048
        } 

        private enum mir_sdr_TransferModeT
        {
          mir_sdr_ISOCH = 0,
          mir_sdr_BULK  = 1
        } 

        private enum mir_sdr_ReasonForReinitT
        {
          mir_sdr_CHANGE_NONE    = 0x00,
          mir_sdr_CHANGE_GR      = 0x01,
          mir_sdr_CHANGE_FS_FREQ = 0x02,
          mir_sdr_CHANGE_RF_FREQ = 0x04,
          mir_sdr_CHANGE_BW_TYPE = 0x08,
          mir_sdr_CHANGE_IF_TYPE = 0x10,
          mir_sdr_CHANGE_LO_MODE = 0x20,
          mir_sdr_CHANGE_AM_PORT = 0x40
        } 

        private enum  mir_sdr_LoModeT
        {
          mir_sdr_LO_Undefined = 0,
          mir_sdr_LO_Auto      = 1,
          mir_sdr_LO_120MHz    = 2,
          mir_sdr_LO_144MHz    = 3,
          mir_sdr_LO_168MHz    = 4
        }

        private enum  mir_sdr_BandT
        {
          mir_sdr_BAND_AM_LO   = 0,
          mir_sdr_BAND_AM_MID  = 1,
          mir_sdr_BAND_AM_HI   = 2,
          mir_sdr_BAND_VHF     = 3,
          mir_sdr_BAND_3       = 4,
          mir_sdr_BAND_X       = 5,
          mir_sdr_BAND_4_5     = 6,
          mir_sdr_BAND_L       = 7
        }

		// added to detect if band boundary has been crossed
		private enum mir_sdr_BandChangeT
		{
			mir_sdr_BAND_UNKNOWN = 0,   //    Unknown band
			mir_sdr_BAND_AM = 1,		//    0 -   60 MHz
			mir_sdr_BAND_VHF = 2,		//   60 -  120 MHz
			mir_sdr_BAND_3   = 3,		//  120 -  250 MHZ
			mir_sdr_BAND_X   = 4,		//  250 -  420 MHz
			mir_sdr_BAND_4_5 = 5,		//  420 - 1000 MHz
			mir_sdr_BAND_L   = 6		// 1000 - 2000 MHZ
		}

		private enum mir_sdr_SetGrModeT
        {
          mir_sdr_USE_SET_GR            = 0,
          mir_sdr_USE_SET_GR_ALT_MODE   = 1,
          mir_sdr_USE_RSP_SET_GR        = 2
        }

		//Low Pass
		//• 12MHz

		//Band Pass
		//• 12 – 30MHz
		//• 30 – 60MHz
		//• 60 – 120MHz
		//• 120 – 250MHz
		//• 250 – 300MHz
		//• 300 – 380MHz
		//• 380 – 420MHz
		//• 420 – 1000MHz

		//High Pass
		//• 1000MHz


		private enum mir_sdr_RSPII_BandT
        {
          mir_sdr_RSPII_BAND_UNKNOWN = 0,	
          mir_sdr_RSPII_BAND_AM_LO   = 1,	// Low pass 12MHz
          mir_sdr_RSPII_BAND_AM_MID  = 2,	//  12 – 30MHz
		  mir_sdr_RSPII_BAND_AM_HI   = 3,   //  30 – 60MHz
		  mir_sdr_RSPII_BAND_VHF     = 4,   //  60 – 120MHz
   		  mir_sdr_RSPII_BAND_3		 = 5,   // 120 – 250MHz
		  mir_sdr_RSPII_BAND_X_LO    = 6,   // 250 – 300MHz
		  mir_sdr_RSPII_BAND_X_MID   = 7,   // 300 – 380MHz
		  mir_sdr_RSPII_BAND_X_HI    = 8,   // 380 – 420MHz
		  mir_sdr_RSPII_BAND_4_5     = 9,   // 420 – 1000MHz
		  mir_sdr_RSPII_BAND_L       = 10   // High pass - 1000MHz
		} 

        public enum mir_sdr_RSPII_AntennaSelectT
        {
          mir_sdr_RSPII_ANTENNA_A = 5,
          mir_sdr_RSPII_ANTENNA_B = 6
        } 

        private enum mir_sdr_AgcControlT
        {
          mir_sdr_AGC_DISABLE  = 0,
          mir_sdr_AGC_100HZ    = 1,
          mir_sdr_AGC_50HZ     = 2,
          mir_sdr_AGC_5HZ      = 3
        }

        private enum mir_sdr_GainMessageIdT : ulong
        {
            mir_sdr_GAIN_MESSAGE_START_ID = 0x80000000,
            mir_sdr_ADC_OVERLOAD_DETECTED = mir_sdr_GAIN_MESSAGE_START_ID + 1,
            mir_sdr_ADC_OVERLOAD_CORRECTED = mir_sdr_GAIN_MESSAGE_START_ID + 2
        } 

        public enum mir_sdr_MinGainReductionT
        {
          mir_sdr_EXTENDED_MIN_GR = 0,
          mir_sdr_NORMAL_MIN_GR   = 20
        }
        
        private struct mir_sdr_DeviceT
        {
           byte SerNo;
           byte DevNm;
           byte hwVer;
           byte devAvail;
        } 

        public struct mir_sdr_GainValuesT
        {
           public float curr;
           public float max;
           public float min;
        }

		// added for clarity in code and avoids magic numbers
		private enum mir_sdr_SyncUpdateT : int
		{
			mir_sdr_IMMEDIATE = 0,
			mir_sdr_SYNCRONOUS = 1
		}

		// added for clarity in code and avoids magic numbers
		private enum mir_sdr_OffsetT
		{
			mir_sdr_OFFSET = 0,
			mir_sdr_ABSOLUTE = 1
		}

		// added for clarity in code and avoids magic numbers
		public enum mir_sdr_DecimationT : uint
		{
			mir_sdr_DISABLED = 0,
			mir_sdr_ENABLED = 1
		}

		// added for clarity in code and avoids magic numbers
		public enum mir_sdr_DecimationFilterT : uint
		{
			mir_sdr_AVERAGING = 0,
			mir_sdr_HALFBAND = 1
		}

		// added for clarity in code and avoids magic numbers
		// Note: API document is ambiguous, text says
		// max factor of 64, params say max factor of 32
		public enum mir_sdr_DecimationFactorT : uint
		{
			mir_sdr_FACTOR_2  = 2,
			mir_sdr_FACTOR_4  = 4,
			mir_sdr_FACTOR_8  = 8,
			mir_sdr_FACTOR_16 = 16,
			mir_sdr_FACTOR_32 = 32,
			mir_sdr_FACTOR_64 = 64
		}

		// added for clarity in code and avoids magic numbers
		public enum mir_sdr_DCmodeT
		{
			mir_sdr_STATIC        = 0,
			mir_sdr_PERIODIC_6ms  = 1,
			mir_sdr_PERIODIC_12ms = 2,
			mir_sdr_PERIODIC_24ms = 3,
			mir_sdr_ONE_SHOT      = 4,
			mir_sdr_CONTINUOUS    = 5
		}

		// added for clarity in code and avoids magic numbers
		public enum mir_sdr_DCmodeSpeedupT
		{
			mir_sdr_DISABLED = 0,
			mir_sdr_ENABLED  = 1
		}




		// includes support for RSP1A, but only tested on a RSPII

		// deprecated function - may be removed from future verisions of the API
		[DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_Init(int gRdB, double fsMHz, double rfMHz, mir_sdr_Bw_MHzT bwType, mir_sdr_If_kHzT ifType, ref int samplesPerPacket);

        // deprecated function - may be removed from future verisions of the API
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_Uninit();

        // deprecated function - may be removed from future verisions of the API
        [DllImport("mir_sdr_api.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "mir_sdr_ReadPacket")]
        private static extern mir_sdr_ErrT mir_sdr_ReadPacket(short[] xi, short[] xq, ref uint firstSampleNum, ref int grChanged, ref int rfChanged, ref int fsChanged);

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetRf(double drfHz, mir_sdr_OffsetT abs, mir_sdr_SyncUpdateT syncUpdate);

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetFs(double dfsHz, int abs, int syncUpdate, int reCal);

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetGr(int gRdB, int abs, int syncUpdate);

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetGrParams(int minimumGr, int lnaGrThreshold);

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetDcMode(mir_sdr_DCmode dcCal, mir_sdr_DCmodeSpeedup speedUp);

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetDcTrackTime(int trackTime);

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetSyncUpdateSampleNum(uint sampleNum);

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetSyncUpdatePeriod(uint period);

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_ApiVersion(ref float version);

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_ResetUpdateFlags(int resetGainUpdate, int resetRfUpdate, int resetFsUpdate);  

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetTransferMode(mir_sdr_TransferModeT mode); 

        // This following function will only operate correctly for the parameters detailed in the table below:
        //
        //      IF freq | Signal BW | Input Sample Rate | Output Sample Rate | Required Decimation Factor 
        //     -------------------------------------------------------------------------------------------
        //       450kHz |   200kHz  |     2000kHz       |       500kHz       |          M=4
        //       450kHz |   300kHz  |     2000kHz       |       500kHz       |          M=4
        //       450kHz |   600kHz  |     2000kHz       |      1000kHz       |          M=2
        //      2048kHz |  1536kHz  |     8192kHz       |      2048kHz       |          M=4
        //
        // If preReset == 1, then the filter state will be reset to 0 before starting the filtering operation.
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_DownConvert(ref short inp, ref short xi, ref short xq, uint samplesPerPacket, mir_sdr_If_kHzT ifType, uint M, uint preReset);

        // deprecated function - may be removed from future verisions of the API
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetParam(uint id, uint value);       // This MAY be called before mir_sdr_Init()

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetPpm(double ppm);                  // This MAY be called before mir_sdr_Init()

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetLoMode(mir_sdr_LoModeT loMode);   // This MUST be called before mir_sdr_Init()/mir_sdr_StreamInit() - otherwise use mir_sdr_Reinit()         5

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetGrAltMode(ref int gRidx, int LNAstate, ref int gRdBsystem, int abs, int syncUpdate);

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_DCoffsetIQimbalanceControl(uint DCenable, uint IQenable);   

        // Valid decimation factors for the following function are 2, 4, 8, 16 or 32 only - API document seems to indicate this can be a maximum of 64
        // Setting wideBandSignal=1 will use a slower filter but minimise the in-band roll-off
		// The function seems to return success even if a non-valid decimation is passed
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_DecimateControl(mir_sdr_Decimation enable, uint decimationFactor, mir_sdr_DecimationFilter wideBandSignal); 

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_AgcControl(mir_sdr_AgcControlT enable, int setPoint_dBfs, int knee_dBfs, uint decay_ms, uint hang_ms, mir_sdr_SyncUpdateT syncUpdate, int LNAstate);

		// mir_sdr_StreamInit() replaces mir_sdr_Init() and sets up a thread (or chain of threads) inside the API which will perform the processing chain (shown below),
		// and then use the callback function to return the data to the calling application/plug-in.
		// Processing chain (in order):
		// mir_sdr_ReadPacket()
		// DCoffsetCorrection()     - LIF mode - enabled by default
		// DownConvert()            - automatically enabled if the parameters shown for mir_sdr_DownConvert() are selected
		// Decimate()               - disabled by default
		// DCoffsetCorrection()     - ZIF mode - enabled by default
		// IQimbalanceCorrection()  - enabled by default
		// Agc()                    - disabled by default
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_StreamInit(ref int gRdB, double fsMHz, double rfMHz, mir_sdr_Bw_MHzT bwType, mir_sdr_If_kHzT ifType, int LNAstate, ref int gRdBsystem, mir_sdr_SetGrModeT setGrMode, ref int samplesPerPacket, mir_sdr_StreamCallback_t StreamCbFn, mir_sdr_GainChangeCallback_t GainChangeCbFn, IntPtr cbContext); 
        
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_StreamUninit(); 
        
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_Reinit(ref int gRdB, double fsMHz, double rfMHz, mir_sdr_Bw_MHzT bwType, mir_sdr_If_kHzT ifType, mir_sdr_LoModeT loMode, int LNAstate, ref int gRdBsystem, mir_sdr_SetGrModeT setGrMode, ref int samplesPerPacket, mir_sdr_ReasonForReinitT reasonForReinit);

        // deprecated function - may be removed from future verisions of the API
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_GetGrByFreq(double rfMHz, ref mir_sdr_BandT band, ref int gRdB, int LNAstate, ref int gRdBsystem, mir_sdr_SetGrModeT setGrMode);
        
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_DebugEnable(uint enable); 
        
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_GetCurrentGain(ref mir_sdr_GainValuesT gainVals);  
        
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_GetDevices(ref mir_sdr_DeviceT devices, ref uint numDevs, uint maxDevs);   

        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_SetDeviceIdx(uint idx);   
        
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern  mir_sdr_ErrT mir_sdr_ReleaseDeviceIdx();   
        
        // Uninit: ver = 0
        // RSPI :  ver = 1
        // RSPII:  ver = 2
		// RSP1A:  don't know what this should return as unable to test - assume 3 ?
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_GetHwVersion(ref byte ver);  
      
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_RSPII_AntennaControl(mir_sdr_RSPII_AntennaSelectT select); 
        
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_RSPII_ExternalReferenceControl(uint output_enable); 
        
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_RSPII_BiasTControl(uint enable); 
        
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_RSPII_RfNotchEnable(uint enable); 
        
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_RSP_SetGr(int gRdB, int LNAstate, mir_sdr_OffsetT abs, mir_sdr_SyncUpdateT syncUpdate);
        
        [DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_RSP_SetGrLimits(mir_sdr_MinGainReductionT minGr);

		// If called after mir_sdr_Init() a call to mir_sdr_Reinit(reasonForReinit = mir_sdr_CHANGE_AM_PORT)
		// is also required to change the port
		[DllImport("mir_sdr_api.dll", CallingConvention=CallingConvention.Cdecl)]
        private static extern mir_sdr_ErrT mir_sdr_AmPortSelect(int port);   

		[DllImport("mir_sdr_api.dll", CallingConvention = CallingConvention.Cdecl)]
		private static extern mir_sdr_ErrT mir_sdr_BiasT(int enable);

		[DllImport("mir_sdr_api.dll", CallingConvention = CallingConvention.Cdecl)]
		private static extern mir_sdr_ErrT mir_sdr_DabNotch(int enable);

		[DllImport("mir_sdr_api.dll", CallingConvention = CallingConvention.Cdecl)]
		private static extern mir_sdr_ErrT mir_sdr_BroadcastNotch(int enable);

	}
}
